/*
*********************************************************************************************************
*                                               uC/OS-II
*                                         The Real-Time Kernel
*
*                        (c) Copyright 1992-1998, Jean J. Labrosse, Plantation, FL
*                                          All Rights Reserved
*
*
*                                       80x86/80x88 Specific code
*                                          LARGE MEMORY MODEL
*
* File : OS_CPU_C.C
* By   : Jean J. Labrosse
*********************************************************************************************************
*/

#define  OS_CPU_GLOBALS
#include "includes.h"
#include "os_kc51.h"
/*
*********************************************************************************************************
*                                        INITIALIZE A TASK'S STACK
*
* Description: This function is called by either OSTaskCreate() or OSTaskCreateExt() to initialize the
*              stack frame of the task being created.  This function is highly processor specific.
*
* Arguments  : task          is a pointer to the task code
*
*              pdata         is a pointer to a user supplied data area that will be passed to the task
*                            when the task first executes.
*
*              ptos          is a pointer to the top of stack.  It is assumed that 'ptos' points to
*                            a 'free' entry on the task stack.  If OS_STK_GROWTH is set to 1 then 
*                            'ptos' will contain the HIGHEST valid address of the stack.  Similarly, if
*                            OS_STK_GROWTH is set to 0, the 'ptos' will contains the LOWEST valid address
*                            of the stack.
*
*              opt           specifies options that can be used to alter the behavior of OSTaskStkInit().
*                            (see uCOS_II.H for OS_TASK_OPT_???).
*
* Returns    : Always returns the location of the new top-of-stack' once the processor registers have
*              been placed on the stack in the proper order.
*
* Note(s)    : Interrupts are enabled when your task starts executing. You can change this by setting the
*              PSW to 0x0002 instead.  In this case, interrupts would be disabled upon task startup.  The
*              application code would be responsible for enabling interrupts at the beginning of the task
*              code.  You will need to modify OSTaskIdle() and OSTaskStat() so that they enable 
*              interrupts.  Failure to do this will make your system crash!
*********************************************************************************************************
*/

extern idata unsigned char Stack[1];

xdata INT16U StkPtr;

void *OSTaskStkInit (void (*task)(void *pd) TASK_REENTRANT, void * vd, void *ptos, INT16U opt) KCREENTRANT
{
    INT8U * stk;

    opt    = opt;                           /* 'opt' is not used, prevent warning                      */
    stk    = (INT8U *) ptos;                /* Load stack pointer                                      */

	stk				-= sizeof(void *);
	*(void**)stk	 = vd;
	stk				-= sizeof(INT16U);
	*(INT16U*)stk	 = (INT16U) task;
	stk				-= sizeof(INT16U);
	*(INT16U*)stk	 = (INT16U) task;
	*--stk			 = PSW;
	*--stk			 = 'A';
	*--stk			 = 'B';
	*--stk			 = 'H';
	*--stk			 = 'L';
	*--stk			 = 0;
/*
	*--stk			 = 1;
	*--stk			 = 2;
	*--stk			 = 3;
*/
	stk				-= sizeof(void *);
	*(void**)stk	 = vd;

	*--stk			 = 4;	
	*--stk			 = 5;
	*--stk			 = 6;
	*--stk			 = 7;


    *--stk			 = (INT8U) Stack-1+8+5+2+2/*+sizeof(void *)*/;

    return ((void *)stk);
}

OS_EXT  OS_TCB      *OSTCBHighRdy;                    /* Pointer to highest priority TCB ready to run  */

void LoadCtx() KCREENTRANT;
void SaveCtx() KCREENTRANT;

extern INT8U xdata * data C_XBP;

void OSStartHighRdy(void) KCREENTRANT
{
    OSTaskSwHook();
	OSRunning++;

	C_XBP=OSTCBHighRdy->OSTCBStkPtr;

	LoadCtx();
}

void C_OSCtxSw(void) KCREENTRANT
{
/*
	Save processor registers;                                              
	Save the current task's stack pointer into the current task's OS_TCB:  
		OSTCBCur->OSTCBStkPtr = Stack pointer;
	Call user definable OSTaskSwHook();                                    
	OSTCBCur  = OSTCBHighRdy;                                              
	OSPrioCur = OSPrioHighRdy;                                             
	Get the stack pointer of the task to resume:                           
		Stack pointer = OSTCBHighRdy->OSTCBStkPtr;
	Restore all processor registers from the new task's stack;             
	Execute a return from interrupt instruction;                           
*/
    OSTCBCur->OSTCBStkPtr = C_XBP;
    
	OSTaskSwHook();
    OSTCBCur  = OSTCBHighRdy;
    OSPrioCur = OSPrioHighRdy;

    C_XBP       = OSTCBCur->OSTCBStkPtr;

    LoadCtx();
}

/*
void C_OSIntCtxSw() KCREENTRANT
{
	Adjust the stack pointer to remove calls to:
        OSIntExit(), 
        OSIntCtxSw() and possibly the push of the processor status word;
	Save the current task's stack pointer into the current task's OS_TCB:
        OSTCBCur->OSTCBStkPtr = Stack pointer;
	Call user definable OSTaskSwHook();
	OSTCBCur  = OSTCBHighRdy;
	OSPrioCur = OSPrioHighRdy;
	Get the stack pointer of the task to resume:
        Stack pointer = OSTCBHighRdy->OSTCBStkPtr;
	Restore all processor registers from the new task's stack;
	Execute a return from interrupt instruction;
}
*/

/*
void OSTickISR() KCREENTRANT
{
    OSIntEnter();
    OSTimeTick();
    OSIntExit();
}
*/

/*$PAGE*/
#if OS_CPU_HOOKS_EN
/*
*********************************************************************************************************
*                                          TASK CREATION HOOK
*
* Description: This function is called when a task is created.
*
* Arguments  : ptcb   is a pointer to the task control block of the task being created.
*
* Note(s)    : 1) Interrupts are disabled during this call.
*********************************************************************************************************
*/
void OSTaskCreateHook (OS_TCB *ptcb) KCREENTRANT
{
    ptcb = ptcb;                       /* Prevent compiler warning                                     */
}


/*
*********************************************************************************************************
*                                           TASK DELETION HOOK
*
* Description: This function is called when a task is deleted.
*
* Arguments  : ptcb   is a pointer to the task control block of the task being deleted.
*
* Note(s)    : 1) Interrupts are disabled during this call.
*********************************************************************************************************
*/
void OSTaskDelHook (OS_TCB *ptcb) KCREENTRANT
{
    ptcb = ptcb;                       /* Prevent compiler warning                                     */
}

/*
*********************************************************************************************************
*                                           TASK SWITCH HOOK
*
* Description: This function is called when a task switch is performed.  This allows you to perform other
*              operations during a context switch.
*
* Arguments  : none
*
* Note(s)    : 1) Interrupts are disabled during this call.
*              2) It is assumed that the global pointer 'OSTCBHighRdy' points to the TCB of the task that
*                 will be 'switched in' (i.e. the highest priority task) and, 'OSTCBCur' points to the 
*                 task being switched out (i.e. the preempted task).
*********************************************************************************************************
*/
void OSTaskSwHook (void) KCREENTRANT
{
}

/*
*********************************************************************************************************
*                                           STATISTIC TASK HOOK
*
* Description: This function is called every second by uC/OS-II's statistics task.  This allows your 
*              application to add functionality to the statistics task.
*
* Arguments  : none
*********************************************************************************************************
*/
void OSTaskStatHook (void) KCREENTRANT
{
}

/*
*********************************************************************************************************
*                                               TICK HOOK
*
* Description: This function is called every tick.
*
* Arguments  : none
*
* Note(s)    : 1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/
void OSTimeTickHook (void) KCREENTRANT
{
}

#endif
